// Actor.cpp: implementation of the CActor class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "Actor.h"

#include "vtkActor.h"
#include "vtkPolyDataMapper.h"
#include "vtkRenderer.h"
#include "vtkClipPolyData.h"
#include "vtkPlane.h"
#include "vtkProperty.h"
#include "vtkRenderWindow.h"
#include "vtkRenderWindowInteractor.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;

#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CActor::CActor()
{
	m_bVisible		= TRUE;
	m_bClip			= FALSE;

	m_mapper		= vtkPolyDataMapper::New();
	m_actor			= vtkActor::New();

	m_mapperClip	= vtkPolyDataMapper::New();
	m_actorClip		= vtkActor::New();

	m_cutPlane		= vtkPlane::New();
	m_clipper		= vtkClipPolyData::New();
	
	m_color			= RGB(255,255,255);			// Initialize 	m_actor->GetProperty()->SetColor(1,1,1) ¦ 


	m_fClipOrigin[0] = 0;
	m_fClipOrigin[1] = 0;
	m_fClipOrigin[2] = 10;
	m_fClipNormal[0] = 1;
	m_fClipNormal[1] = 0;
	m_fClipNormal[2] = 0;
}

CActor::~CActor()
{
	m_actor     ->Delete();
	m_mapper    ->Delete();

	m_actorClip ->Delete();
	m_mapperClip->Delete();

	m_cutPlane  ->Delete();
	m_clipper   ->Delete();
}

unsigned long CActor::GetActorMTime()
{
	return m_actor->GetMTime();
	//return m_actor->GetRedrawMTime();
}

void CActor::Initialize()
{
	m_mapper    ->SetScalarVisibility(FALSE);
	m_actor     ->SetMapper(m_mapper);
	m_actor     ->GetProperty()->SetColor(1,1,1);		//  m_color=RGB(255,255,255) ¦ 

	//m_actor     ->GetProperty()->SetAmbientColor(.5,.5,.5);
	//m_actor     ->GetProperty()->SetDiffuseColor(.1,-2,.4);
	m_actor     ->GetProperty()->SetSpecularColor(1,1,1);


	m_mapperClip->SetScalarVisibility(FALSE);
	m_actorClip ->SetMapper(m_mapperClip);
	m_actorClip ->GetProperty()->SetColor(1,1,1);

	m_mapperClip->SetInputConnection(m_clipper->GetClippedOutputPort());

	m_clipper->GenerateClipScalarsOff();
	m_clipper->SetClipFunction(m_cutPlane);
	m_clipper->GenerateClippedOutputOn();
	m_clipper->ReleaseDataFlagOn();
}

float CActor::GetOpacity()
{
	return m_actor->GetProperty()->GetOpacity();
}

void CActor::SetOpacity(double opacity)
{
	if(opacity > 1.0f) opacity = 1.0f;
	if(opacity < 0.0f) opacity = 0.0f;

	m_actor    ->GetProperty()->SetOpacity(opacity);
	m_actorClip->GetProperty()->SetOpacity(opacity);
}

void CActor::SetColor(COLORREF color)
{
	m_color = color;

	double cr[3];
	cr[0] = GetRValue(color) / 255.0;
	cr[1] = GetGValue(color) / 255.0;
	cr[2] = GetBValue(color) / 255.0;

	m_actor    ->GetProperty()->SetColor(cr);
//	m_actor    ->GetProperty()->SetDiffuseColor(cr);
	m_actor    ->GetProperty()->SetSpecular(.3);
	m_actor    ->GetProperty()->SetSpecularPower(20);

	m_actorClip->GetProperty()->SetColor(cr);
//	m_actorClip->GetProperty()->SetDiffuseColor(cr);
	m_actorClip->GetProperty()->SetSpecular(.3);
	m_actorClip->GetProperty()->SetSpecularPower(20);

}

void CActor::SetColor(double crR, double crG, double crB)
{
	m_color = RGB(
		int(crR * 255),
		int(crG * 255),
		int(crB * 255)	);

	SetColor(m_color);
}



void CActor::AddToRenderer(vtkRenderer* ren)
{
	/***************************************************
	* CActorü renderer ߰ϰ,
	* renderer Ʈ ߰
	***************************************************/

	m_arrAddedRenderer.Add(ren);

	if(m_bClip == TRUE)
	{
		ren->AddActor(m_actorClip);
		ren->GetRenderWindow()->GetInteractor()->Render();
	}
	else
	{
		ren->AddActor(m_actor);
		ren->GetRenderWindow()->GetInteractor()->Render();
	}
}

int CActor::FindRendererIndex(vtkRenderer* ren)
{
	int count = m_arrAddedRenderer.GetSize();
	for(int i = 0; i < count; i++)
	{
		if(m_arrAddedRenderer[i] == ren)
			return i;
	}
	return -1;
}


void CActor::RemoveFromRenderer(vtkRenderer* ren)
{
	/***************************************************
	* CActorü renderer ϰ,
	* renderer Ʈ 
	***************************************************/

	if(m_bClip == TRUE)
	{
		ren->RemoveActor(m_actorClip);
	}
	else
	{
		ren->RemoveActor(m_actor);
	}


	int iIndex = FindRendererIndex(ren);
	if(iIndex != -1)
	{
		m_arrAddedRenderer.RemoveAt(iIndex);
		return;
	}
	
	ASSERT(FALSE);
}

void CActor::RemoveVtkActorFromAllRenderer(vtkActor* actor)
{
	/*******************************************************
	* vtkActor ϵ  rendererκ Ѵ
	*******************************************************/

	int count = m_arrAddedRenderer.GetSize();
	for(int i = 0; i < count; i++)
	{
		m_arrAddedRenderer[i]->RemoveActor(actor);
	}

}

void CActor::AddVtkActorToAllRenderer(vtkActor* actor)
{
	/*******************************************************
	* vtkActor ϵ  renderer ߰Ѵ
	*******************************************************/

	int count = m_arrAddedRenderer.GetSize();
	for(int i = 0; i < count; i++)
	{
		m_arrAddedRenderer[i]->AddActor(actor);
	}

}


BOOL CActor::GetVisible()
{
	return m_bVisible;
}

void CActor::SetVisible(BOOL bVisible)
{
	if(m_bVisible == bVisible) return;
	m_bVisible = bVisible;

	m_actor    ->SetVisibility(m_bVisible);
	m_actorClip->SetVisibility(bVisible);
}


BOOL CActor::IsActorIdentical(vtkActor *actor)
{
	return (m_actor == actor) || (m_actorClip == actor);
}

COLORREF CActor::GetColor()
{
	return m_color;
}

void CActor::Clip(BOOL bOn, double* normal, double* origin)
{
	m_bClip = bOn;

	if(bOn == TRUE)
	{
		if(origin != NULL) m_cutPlane->SetOrigin(origin[0],origin[1],origin[2]);
		if(normal != NULL) m_cutPlane->SetNormal(normal[0], normal[1], normal[2]);
		
		RemoveVtkActorFromAllRenderer(m_actor);
		AddVtkActorToAllRenderer(m_actorClip);
	}
	else
	{
		RemoveVtkActorFromAllRenderer(m_actorClip);
		AddVtkActorToAllRenderer(m_actor);
	}

}

void CActor::SetInputForMapper(vtkAlgorithmOutput *polyDataInput)
{
	m_mapper ->SetInputConnection(polyDataInput);
	m_mapper->Update();
	m_clipper->SetInputConnection(polyDataInput);
	m_clipper->Update();
}

void CActor::SetScalarVisibility(BOOL bOn)
{
	m_mapper    ->SetScalarVisibility(bOn);
	m_mapperClip->SetScalarVisibility(bOn);
}

void CActor::SetTexture(vtkTexture *texture)
{
	m_actor    ->SetTexture(texture);
	m_actorClip->SetTexture(texture);
}

BOOL CActor::IsClipping()
{
	return m_bClip;
}

void CActor::GetClipInfo(double *normal, double *origin)
{
	memcpy(origin, m_fClipOrigin, sizeof(float)*3);
	memcpy(normal, m_fClipNormal, sizeof(float)*3);
}
