// vtkCrossLineSource.cpp: implementation of the vtkCrossLineSource class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "vtkCrossLineSource.h"

#include "vtkCellArray.h"
#include "vtkFloatArray.h"
#include "vtkInformation.h"
#include "vtkInformationVector.h"
#include "vtkObjectFactory.h"
#include "vtkPointData.h"
#include "vtkPoints.h"
#include "vtkPolyData.h"
#include "vtkStreamingDemandDrivenPipeline.h"
#include "vtkVector.h"
#include "vtkVectorOperators.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

vtkCrossLineSource::vtkCrossLineSource()
{
	for(int i=0;i<6;i++)
		m_bShowShape[i] = TRUE;
	
	this->m_fLength = 10;

	for(int i=0;i<3;i++)
		m_pos[i] = 0.0f;
}

vtkCrossLineSource::~vtkCrossLineSource()
{

}

//------------------------------------------------------------------------------
vtkCrossLineSource* vtkCrossLineSource::New()
{
	// First try to create the object from the vtkObjectFactory
	vtkObject* ret = vtkObjectFactory::CreateInstance("vtkCrossLineSource");
	if(ret)
	{
		return (vtkCrossLineSource*)ret;
	}
	// If the factory was unable to create the object, then create it here.
	return new vtkCrossLineSource;
}

	void vtkCrossLineSource::SetLineLength(float fLength)
{
	ASSERT(fLength > 0.0f);
	m_fLength = fLength;
	Modified();
}

void vtkCrossLineSource::SetPosition(float* pos)
{
	m_pos[0] = pos[0];
	m_pos[1] = pos[1];
	m_pos[2] = pos[2];
	Modified();
}

void vtkCrossLineSource::SetShowShape(BOOL* bShowShape)
{
	for(int i = 0; i < 6; i++)
		m_bShowShape[i] = bShowShape[i];

	Modified();
}


void vtkCrossLineSource::Execute()
{
	vtkPoints*		newPoints		= vtkPoints::New();
	vtkCellArray*	newLines		= vtkCellArray::New();


	newPoints->InsertPoint(0, m_pos[0]+m_fLength,	m_pos[1],			m_pos[2]);				// +x
	newPoints->InsertPoint(1, m_pos[0]-m_fLength,	m_pos[1],			m_pos[2]);				// -x
	newPoints->InsertPoint(2, m_pos[0],				m_pos[1]+m_fLength, m_pos[2]);				// +y
	newPoints->InsertPoint(3, m_pos[0],				m_pos[1]-m_fLength, m_pos[2]);				// -y
	newPoints->InsertPoint(4, m_pos[0],				m_pos[1],			m_pos[2]+m_fLength);	// +z
	newPoints->InsertPoint(5, m_pos[0],				m_pos[1],			m_pos[2]-m_fLength);	// -z
	newPoints->InsertPoint(6, m_pos[0],				m_pos[1],			m_pos[2]);				// center


	for(int i = 0; i < 6; i++)
	{
		if(m_bShowShape[i] == TRUE)
		{
			newLines->InsertNextCell(2);
			newLines->InsertCellPoint(6);
			newLines->InsertCellPoint(i);
		}
	}

	vtkPolyData* output = GetOutput();
	output->SetPoints(newPoints);
	output->SetLines(newLines);

	newPoints->Delete();
	newLines->Delete();	
	
}

int vtkCrossLineSource::RequestInformation(vtkInformation* vtkNotUsed(request),
	vtkInformationVector** vtkNotUsed(inputVector), vtkInformationVector* outputVector)
{
	vtkInformation* outInfo = outputVector->GetInformationObject(0);
	outInfo->Set(CAN_HANDLE_PIECE_REQUEST(), 1);
	return 1;
}

int vtkCrossLineSource::RequestData(vtkInformation* vtkNotUsed(request),
	vtkInformationVector** vtkNotUsed(inputVector), vtkInformationVector* outputVector)
{
	vtkPolyData* output = this->GetOutput();

	vtkPoints* newPoints = vtkPoints::New();
	vtkCellArray* newLines = vtkCellArray::New();

	newPoints->InsertPoint(0, m_pos[0] + m_fLength, m_pos[1], m_pos[2]);				// +x
	newPoints->InsertPoint(1, m_pos[0] - m_fLength, m_pos[1], m_pos[2]);				// -x
	newPoints->InsertPoint(2, m_pos[0], m_pos[1] + m_fLength, m_pos[2]);				// +y
	newPoints->InsertPoint(3, m_pos[0], m_pos[1] - m_fLength, m_pos[2]);				// -y
	newPoints->InsertPoint(4, m_pos[0], m_pos[1], m_pos[2] + m_fLength);				// +z
	newPoints->InsertPoint(5, m_pos[0], m_pos[1], m_pos[2] - m_fLength);				// -z
	newPoints->InsertPoint(6, m_pos[0], m_pos[1], m_pos[2]);							// center


	for (int i = 0; i < 6; i++)
	{
		if (m_bShowShape[i] == TRUE)
		{
			newLines->InsertNextCell(2);
			newLines->InsertCellPoint(6);
			newLines->InsertCellPoint(i);
		}
	}

	output->SetPoints(newPoints);
	output->SetLines(newLines);

	newPoints->Delete();
	newLines->Delete();

	return 1;
}

//----------------------------------------------------------------------------
void vtkCrossLineSource::PrintSelf(ostream& os, vtkIndent indent)
{
	vtkPolyDataAlgorithm::PrintSelf(os,indent);
}

